@@ -21,22 +21,23 @@ Paul Licameli split from TrackPanel.cpp
 #include "../../../widgets/BasicMenu.h"
 #include "AllThemeResources.h"
 #include "../../../HitTestResult.h"
+#include "PendingTracks.h"
 #include "Project.h"
 #include "ProjectHistory.h"
 #include "ProjectNumericFormats.h"
 #include "ProjectRate.h"
-#include "../../../ProjectWindow.h"
 #include "../../../ProjectWindows.h"
 #include "../../../RefreshCode.h"
 #include "SyncLock.h"
 #include "Theme.h"
 #include "../../../TrackArt.h"
 #include "../../../TrackArtist.h"
-#include "../../../TrackPanelAx.h"
+#include "TrackFocus.h"
 #include "../../../TrackPanel.h"
 #include "../../../TrackPanelMouseEvent.h"
 #include "UndoManager.h"
 #include "ViewInfo.h"
+#include "Viewport.h"
 #include "AudacityTextEntryDialog.h"
 #include "wxWidgetsWindowPlacement.h"
 
@@ -96,8 +97,8 @@ void LabelTrackView::Index::SetModified(bool modified)
    mModified = modified;
 }
 
-LabelTrackView::LabelTrackView( const std::shared_ptr<Track> &pTrack )
-   : CommonTrackView{ pTrack }
+LabelTrackView::LabelTrackView(const std::shared_ptr<Channel> &pChannel)
+   : CommonChannelView{ pChannel }
 {
    ResetFont();
    CreateCustomGlyphs();
@@ -112,13 +113,15 @@ LabelTrackView::~LabelTrackView()
 {
 }
 
-void LabelTrackView::Reparent( const std::shared_ptr<Track> &parent )
+void LabelTrackView::Reparent(
+   const std::shared_ptr<Track> &parent, size_t iChannel)
 {
+   assert(iChannel == 0);
    auto oldParent = FindLabelTrack();
    auto newParent = track_cast<LabelTrack*>(parent.get());
    if (oldParent.get() != newParent)
       BindTo( newParent );
-   CommonTrackView::Reparent( parent );
+   CommonChannelView::Reparent(parent, iChannel);
 }
 
 void LabelTrackView::BindTo( LabelTrack *pParent )
@@ -140,12 +143,12 @@ void LabelTrackView::BindTo( LabelTrack *pParent )
    });
 }
 
-void LabelTrackView::CopyTo( Track &track ) const
+void LabelTrackView::CopyTo(Track &track, size_t iChannel) const
 {
-   TrackView::CopyTo( track );
-   auto &other = TrackView::Get( track );
-
-   if ( const auto pOther = dynamic_cast< const LabelTrackView* >( &other ) ) {
+   assert(iChannel == 0);
+   ChannelView::CopyTo(track, iChannel);
+   auto &other = ChannelView::Get(*track.GetChannel(0));
+   if (const auto pOther = dynamic_cast<const LabelTrackView*>(&other)) {
       pOther->mNavigationIndex = mNavigationIndex;
       pOther->mInitialCursorPos = mInitialCursorPos;
       pOther->mCurrentCursorPos = mCurrentCursorPos;
@@ -154,19 +157,19 @@ void LabelTrackView::CopyTo( Track &track ) const
    }
 }
 
-LabelTrackView &LabelTrackView::Get( LabelTrack &track )
+LabelTrackView &LabelTrackView::Get(LabelTrack &track)
 {
-   return static_cast< LabelTrackView& >( TrackView::Get( track ) );
+   return static_cast<LabelTrackView&>(ChannelView::Get(track));
 }
 
-const LabelTrackView &LabelTrackView::Get( const LabelTrack &track )
+const LabelTrackView &LabelTrackView::Get(const LabelTrack &track)
 {
-   return static_cast< const LabelTrackView& >( TrackView::Get( track ) );
+   return static_cast<const LabelTrackView&>(ChannelView::Get(track));
 }
 
 std::shared_ptr<LabelTrack> LabelTrackView::FindLabelTrack()
 {
-   return std::static_pointer_cast<LabelTrack>( FindTrack() );
+   return FindChannel<LabelTrack>();
 }
 
 std::shared_ptr<const LabelTrack> LabelTrackView::FindLabelTrack() const
@@ -257,7 +260,7 @@ void LabelTrackView::ResetFont()
 {
    mFontHeight = -1;
    wxString facename = gPrefs->Read(wxT("/GUI/LabelFontFacename"), wxT(""));
-   int size = gPrefs->Read(wxT("/GUI/LabelFontSize"), DefaultFontSize);
+   int size = gPrefs->Read(wxT("/GUI/LabelFontSize"), static_cast<int>(DefaultFontSize));
    msFont = GetFont(facename, size);
 }
 
@@ -436,10 +439,7 @@ void LabelTrackView::ComputeLayout(const wxRect & r, const ZoomInfo &zoomInfo) c
    // allowed to be obscured by the text].
    const int xExtra= (3 * mIconWidth)/2;
 
-   bool bAvoidName = false;
    const int nRows = wxMin((r.height / yRowHeight) + 1, MAX_NUM_ROWS);
-   if( nRows > 2 )
-      bAvoidName = gPrefs->ReadBool(wxT("/GUI/ShowTrackNameInWaveform"), false);
    // Initially none of the rows have been used.
    // So set a value that is less than any valid value.
    {
@@ -479,21 +479,6 @@ void LabelTrackView::ComputeLayout(const wxRect & r, const ZoomInfo &zoomInfo) c
       // IF we found such a row THEN record a valid position.
       if( iRow<nRows )
       {
-         // Logic to ameliorate case where first label is under the 
-         // (on track) track name.  For later labels it does not matter
-         // as we can scroll left or right and/or zoom.
-         // A possible alternative idea would be to (instead) increase the 
-         // translucency of the track name, when the mouse is inside it.
-         if( (i==0 ) && (iRow==0) && bAvoidName ){
-            // reserve some space in first row.
-            // reserve max of 200px or t1, or text box right edge.
-            const int x2 = zoomInfo.TimeToPosition(0.0, r.x) + 200;
-            xUsed[iRow]=x+labelStruct.width+xExtra;
-            if( xUsed[iRow] < x1 ) xUsed[iRow]=x1;
-            if( xUsed[iRow] < x2 ) xUsed[iRow]=x2;
-            iRow=1;
-         }
-
          // Possibly update the number of rows actually used.
          if( iRow >= nRowsUsed )
             nRowsUsed=iRow+1;
@@ -777,12 +762,13 @@ namespace {
 /// Draw calls other functions to draw the LabelTrack.
 ///   @param  dc the device context
 ///   @param  r  the LabelTrack rectangle.
-void LabelTrackView::Draw
-( TrackPanelDrawingContext &context, const wxRect & r ) const
+void LabelTrackView::Draw(TrackPanelDrawingContext &context, const wxRect & r)
+const
 {
    auto &dc = context.dc;
    const auto artist = TrackArtist::Get( context );
    const auto &zoomInfo = *artist->pZoomInfo;
+   const auto &pendingTracks = *artist->pPendingTracks;
 
    auto pHit = findHit( artist->parent );
 
@@ -792,13 +778,15 @@ void LabelTrackView::Draw
    if (mFontHeight == -1)
       calculateFontHeight(dc);
 
-   const auto pTrack = std::static_pointer_cast< const LabelTrack >(
-      FindTrack()->SubstitutePendingChangedTrack());
-   const auto &mLabels = pTrack->GetLabels();
+   if (!FindChannel())
+      return;
+   const auto &track = static_cast<const LabelTrack&>(
+      pendingTracks.SubstitutePendingChangedChannel(*FindChannel()));
+   const auto &mLabels = track.GetLabels();
 
-   TrackArt::DrawBackgroundWithSelection( context, r, pTrack.get(),
+   TrackArt::DrawBackgroundWithSelection(context, r, track,
       AColor::labelSelectedBrush, AColor::labelUnselectedBrush,
-      SyncLock::IsSelectedOrSyncLockSelected(pTrack.get()) );
+      SyncLock::IsSelectedOrSyncLockSelected(track));
 
    wxCoord textWidth, textHeight;
 
@@ -851,7 +839,8 @@ void LabelTrackView::Draw
 #ifdef EXPERIMENTAL_TRACK_PANEL_HIGHLIGHTING
       bool highlightTrack = false;
       auto target = dynamic_cast<LabelTextHandle*>(context.target.get());
-      highlightTrack = target && target->GetTrack().get() == this;
+      highlightTrack = target &&
+         target->FindTrack().get() == FindTrack().get();
 #endif
       int i = -1; for (const auto &labelStruct : mLabels) { ++i;
          bool highlight = false;
@@ -926,7 +915,7 @@ void LabelTrackView::Draw(
 {
    if ( iPass == TrackArtist::PassTracks )
       Draw( context, rect );
-   CommonTrackView::Draw( context, rect, iPass );
+   CommonChannelView::Draw(context, rect, iPass);
 }
 
 /// uses GetTextExtent to find the character position
@@ -1455,7 +1444,7 @@ unsigned LabelTrackView::KeyDown(
    // Make sure caret is in view
    int x;
    if (CalcCursorX( *project, &x ))
-      ProjectWindow::Get( *project ).ScrollIntoView(x);
+      Viewport::Get(*project).ScrollIntoView(x);
 
    // If selection modified, refresh
    // Otherwise, refresh track display if the keystroke was handled
@@ -1684,7 +1673,7 @@ bool LabelTrackView::DoKeyDown(
       case WXK_NUMPAD_ENTER:
       case WXK_TAB:
          if (mRestoreFocus >= 0) {
-            auto track = *TrackList::Get( project ).Any()
+            auto track = *TrackList::Get(project).Any()
                .begin().advance(mRestoreFocus);
             if (track)
                TrackFocus::Get( project ).Set(track);
@@ -1762,7 +1751,7 @@ bool LabelTrackView::DoKeyDown(
                mInitialCursorPos = mCurrentCursorPos;
                //Set the selection region to be equal to the selection bounds of the tabbed-to label.
                newSel = labelStruct.selectedRegion;
-               ProjectWindow::Get(project).ScrollIntoView(labelStruct.selectedRegion.t0());
+               Viewport::Get(project).ScrollIntoView(labelStruct.selectedRegion.t0());
                // message for screen reader
                /* i18n-hint:
                   String is replaced by the name of a label,
@@ -1909,7 +1898,7 @@ void LabelTrackView::ShowContextMenu( AudacityProject &project )
 
    // Bug 2044.  parent can be nullptr after a context switch.
    if( !parent )
-      parent = &GetProjectFrame( project );
+      parent = &GetProjectFrame(project);
 
    if( parent )
    {
@@ -2077,7 +2066,8 @@ int LabelTrackView::GetLabelIndex(double t, double t1)
 
 // restoreFocus of -1 is the default, and sets the focus to this label.
 // restoreFocus of -2 or other value leaves the focus unchanged.
-// restoreFocus >= 0 will later cause focus to move to that track.
+// restoreFocus >= 0 will later cause focus to move to that track (counting
+// tracks, not channels)
 int LabelTrackView::AddLabel(const SelectedRegion &selectedRegion,
                          const wxString &title, int restoreFocus)
 {
@@ -2087,9 +2077,9 @@ int LabelTrackView::AddLabel(const SelectedRegion &selectedRegion,
    return pos;
 }
 
-void LabelTrackView::OnLabelAdded( const LabelTrackEvent &e )
+void LabelTrackView::OnLabelAdded(const LabelTrackEvent &e)
 {
-   if ( e.mpTrack.lock() != FindTrack() )
+   if (e.mpTrack.lock() != FindLabelTrack())
       return;
 
    const auto &title = e.mTitle;
@@ -2108,9 +2098,9 @@ void LabelTrackView::OnLabelAdded( const LabelTrackEvent &e )
       mRestoreFocus = -2;
 }
 
-void LabelTrackView::OnLabelDeleted( const LabelTrackEvent &e )
+void LabelTrackView::OnLabelDeleted(const LabelTrackEvent &e)
 {
-   if ( e.mpTrack.lock() != FindTrack() )
+   if (e.mpTrack.lock() != FindLabelTrack())
       return;
 
    auto index = e.mFormerPosition;
@@ -2126,9 +2116,9 @@ void LabelTrackView::OnLabelDeleted( const LabelTrackEvent &e )
       --mTextEditIndex;//NB: Keep cursor selection region
 }
 
-void LabelTrackView::OnLabelPermuted( const LabelTrackEvent &e )
+void LabelTrackView::OnLabelPermuted(const LabelTrackEvent &e)
 {
-   if ( e.mpTrack.lock() != FindTrack() )
+   if (e.mpTrack.lock() != FindLabelTrack())
       return;
 
    auto former = e.mFormerPosition;
@@ -2146,12 +2136,12 @@ void LabelTrackView::OnLabelPermuted( const LabelTrackEvent &e )
    fix(mTextEditIndex);
 }
 
-void LabelTrackView::OnSelectionChange( const LabelTrackEvent &e )
+void LabelTrackView::OnSelectionChange(const LabelTrackEvent &e)
 {
-   if ( e.mpTrack.lock() != FindTrack() )
+   if (e.mpTrack.lock() != FindLabelTrack())
       return;
 
-   if (!FindTrack()->GetSelected())
+   if (!FindLabelTrack()->GetSelected())
    {
        SetNavigationIndex(-1);
        ResetTextSelection();
@@ -2276,7 +2266,7 @@ void LabelTrackView::DoEditLabels
       freqFormat = formats.GetFrequencySelectionFormatName();
    auto &tracks = TrackList::Get( project );
    auto &viewInfo = ViewInfo::Get( project );
-   auto &window = ProjectWindow::Get( project );
+   auto &window = GetProjectFrame(project);
 
    LabelDialog dlg(&window, project, &tracks,
                    lt, index,
@@ -2309,7 +2299,7 @@ int LabelTrackView::DialogForLabelName(
       - 39;
    position.y += 2;  // just below the bottom of the track
    position = trackPanel.ClientToScreen(position);
-   auto &window = GetProjectFrame( project );
+   auto &window = GetProjectFrame(project);
    AudacityTextEntryDialog dialog{ &window,
       XO("Name:"),
       XO("New label"),
@@ -2340,14 +2330,15 @@ int LabelTrackView::DialogForLabelName(
    return status;
 }
 
-using DoGetLabelTrackView = DoGetView::Override< LabelTrack >;
+using DoGetLabelTrackView = DoGetView::Override<LabelTrack>;
 DEFINE_ATTACHED_VIRTUAL_OVERRIDE(DoGetLabelTrackView) {
-   return [](LabelTrack &track) {
-      return std::make_shared<LabelTrackView>( track.SharedPointer() );
+   return [](LabelTrack &track, size_t) {
+      return std::make_shared<LabelTrackView>(
+         track.SharedPointer<LabelTrack>());
    };
 }
 
-std::shared_ptr<TrackVRulerControls> LabelTrackView::DoGetVRulerControls()
+std::shared_ptr<ChannelVRulerControls> LabelTrackView::DoGetVRulerControls()
 {
    return
       std::make_shared<LabelTrackVRulerControls>( shared_from_this() );
